/*
 *  File:  TriangleTest.java
 *
 *  Draw a Sierpinski triangle (a fractal)
 *
 *  Copyright:  Northeast Parallel Architectures Center
 *  
 */

import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Point;

public class TriangleTest extends Applet {

   // create a reference to a triangle:
   private Triangle triangle;
   
   // create references to the vertices of a triangle:
   private Point v0, v1, v2;
   
   // maximum depth of recursion:
   public int maxDepth;      
   
   public void init() {
   
      // triangle vertices:
      v0 = new Point( 150,  25 );
      v1 = new Point( 294, 275 );
      v2 = new Point(   6, 275 );
      
      // get 'maxDepth' parameter from the HTML document:
      String maxDepthStr = getParameter( "maxDepth" );
      maxDepth = string2int( maxDepthStr );

   }

   public int string2int( String s ) {
      if ( s == null ) {
         return 3;  // default
      } else {
         return Integer.parseInt( s );
      }
   }

   
   public void paint( Graphics g ) {

      // draw initial triangle:
      triangle = new Triangle( v0, v1, v2 );
      g.setColor( Color.black );
      triangle.fill( g );
      
      // divide and fill:
      g.setColor( Color.white );
      divideAndFill( g, 0 );

   }
   
   private void divideAndFill( Graphics g, int depth ) {
      
      if ( depth < maxDepth ) {
          
          // get the vertices of the current triangle:
          Point p0 = new Point( triangle.xpoints[0],
                                triangle.ypoints[0]  );
          Point p1 = new Point( triangle.xpoints[1],
                                triangle.ypoints[1]  );
          Point p2 = new Point( triangle.xpoints[2],
                                triangle.ypoints[2]  );
          
          // compute the midpoint of each side:
          Point p01 = midpoint( p0, p1 );
          Point p02 = midpoint( p0, p2 );
          Point p12 = midpoint( p1, p2 );
          
          // fill the inner triangle:
          ( new Triangle( p01, p02, p12 ) ).fill( g );
          
          // recurse on three subtriangles:
          triangle = new Triangle( p0, p01, p02 );
          divideAndFill( g, depth + 1 );
          triangle = new Triangle( p1, p01, p12 );
          divideAndFill( g, depth + 1 );
          triangle = new Triangle( p2, p02, p12 );
          divideAndFill( g, depth + 1 );
          
      }
       
   }
   
   private Point midpoint( Point p, Point q ) {

      return new Point( (int) Math.round( (p.x + q.x)/2.0 ), 
                        (int) Math.round( (p.y + q.y)/2.0 )  );

   }
   
}
